﻿// PSoC1 term ver 2.01
// Author: Junichi Akita
// Creative Commons License CC BY-NC-ND
//　http://creativecommons.jp/licenses/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Management;

// get COM port name list
//  http://axion.sakura.ne.jp/blog/index.php?UID=1278233577
//  http://rapislazuli.dyndns.org/d/20101225.html

namespace PSoC1term
{
    public partial class Form1 : Form
    {
        static string fname;
        static StreamReader sw = null;
        static int LastBlock = 215; // 215 for 27 / 463 for 29
        static bool fTermBinary = false;
        static bool fWriting = false;
        public Form1()
        {
            InitializeComponent();
            labelConnectStatus.Text = "Disconnected";
            textBoxTerm.Enabled = false;
            textBoxTermR.Enabled = false;
            buttonTermBinarySend.Enabled = false;
            comboBoxBaudTerm.Items.Add("9600");
            comboBoxBaudTerm.Items.Add("19200");
            comboBoxBaudTerm.Items.Add("38400");
            comboBoxBaudTerm.Items.Add("57600");
            comboBoxBaudTerm.Items.Add("115200");
            comboBoxBaudTerm.SelectedIndex = 0;
            comboBoxBaudWrite.Items.Add("9600");
            comboBoxBaudWrite.Items.Add("19200");
            comboBoxBaudWrite.Items.Add("38400");
            comboBoxBaudWrite.Items.Add("57600");
            comboBoxBaudWrite.Items.Add("115200");
            comboBoxBaudWrite.SelectedIndex = 4;
            SortedList<int, string> PortsTable = new SortedList<int, string>();
            ManagementClass mcW32SerPort = new ManagementClass("Win32_PnPEntity");
            foreach (ManagementObject s in mcW32SerPort.GetInstances())
            {
                string name = (string)s.GetPropertyValue("Name");
                if (name != null)
                {
                    if (name.IndexOf("(COM") >= 0)
                    {
                        string port, desc;
                        int p, pos;
                        pos = name.IndexOf("(COM");
                        if (pos > 1)
                        {
                            string ftserial;
                            port = name.Substring(pos);
                            port = port.Replace("(COM", "");
                            port = port.Replace(")", "");
                            p = int.Parse(port);

                            ftserial = (string)s.GetPropertyValue("DeviceID");
                            if (ftserial.IndexOf("FTDIBUS") >= 0)
                            {
                                ftserial = ftserial.Replace(@"FTDIBUS\VID_0403+PID_6001+", "");
                                ftserial = ftserial.Replace(@"A\0000", "");
                            }
                            else
                            {
                                ftserial = "";
                            }
                            if (ftserial.Length > 0)
                            {
                                desc = name.Substring(0, pos - 1) + "(" + ftserial + ")";
                            }
                            else
                            {
                                desc = name.Substring(0, pos - 1);
                            }
                            PortsTable.Add(p, desc);
                        }
                    }
                }
            }
            comboBoxCOMport.Items.Clear();
            foreach (KeyValuePair<int, string> ComPort in PortsTable)
            {
                comboBoxCOMport.Items.Add("COM"+ComPort.Key+" : " + ComPort.Value);
            }
            if (comboBoxCOMport.Items.Count == 0)
            {
                MessageBox.Show("No COM ports available.");
            }
            else
            {
                comboBoxCOMport.SelectedIndex = 0;
            }
            ViewResize();
        }

        private void serialOpen(string s)
        {
            int baud = Convert.ToInt32(s);
            try
            {
                if (serialPort.IsOpen == true) serialPort.Close();
                serialPort.BaudRate = baud;
                if (serialPort.PortName != "") serialPort.Open();
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message);
                buttonOpenFile.Enabled = false;
                buttonWrite.Enabled = false;
                buttonTerm.Enabled = false;
                textBoxTermBinarySend.Enabled = false;
                buttonTermBinarySend.Enabled = false;
            }
            finally
            {
                if (serialPort.IsOpen == true)
                {
                    buttonTerm.Enabled = true;
                    buttonOpenFile.Enabled = true;
                }
            }
        }

        private void buttonQuit_Click(object sender, EventArgs e)
        {
            if (serialPort.IsOpen == true) serialPort.Close();
            Close();
        }

        private void buttonTerm_Click(object sender, EventArgs e)
        {
            if (textBoxTerm.Enabled == true)
            {
                serialPort.Close();
                textBoxTerm.Enabled = false;
                textBoxTermR.Enabled = false;
                buttonTerm.Text = "Term Open";
                labelConnectStatus.Text = "Disconnected";
                buttonTermBinarySend.Enabled = false;
                textBoxTermBinarySend.Enabled = false;
            }
            else
            {
                serialOpen(comboBoxBaudTerm.SelectedItem.ToString());
                if (serialPort.IsOpen == true)
                {
                    textBoxTerm.Enabled = true;
                    textBoxTermR.Enabled = true;
                    buttonTerm.Text = "Term Close";
                    textBoxTerm.ForeColor = Color.Black;
                    textBoxTerm.Text = "";
                    labelConnectStatus.Text = "Terminal Open";
                    if (checkBoxTermBinary.Checked == true) buttonTermBinarySend.Enabled = true;
                    else buttonTermBinarySend.Enabled = false;
                    nb = 0;
                    if (checkBoxTermBinary.Checked == true)
                    {
                        buttonTermBinarySend.Enabled = true;
                        textBoxTermBinarySend.Enabled = true;
                        textBoxTermBinarySend.Focus();
                    }
                }
            }
        }

        private void buttonOpenFile_Click(object sender, EventArgs e)
        {
            openFileDialog.Filter = "Intex Hex(*.hex)|*.hex|" + "All files(*.*)|*.*";
            if (openFileDialog.ShowDialog() != DialogResult.OK)
            {
                return;
            }
            fname = openFileDialog.FileName;
            labelHexFilename.Text = fname;
            buttonWrite.Enabled = true;

        }

        static int nb = 0;
        private void buttonWrite_Click(object sender, EventArgs e)
        {
            string s, ts;
            int b, i;
            if (buttonTerm.Enabled == true)
            {
                serialPort.Close();
                textBoxTerm.Enabled = false;
                buttonTerm.Text = "Term Open";
                labelConnectStatus.Text = "Disconnected";
                buttonTermBinarySend.Enabled = false;
            }
            if (radioButton27443.Checked == true) LastBlock = 215;
            else LastBlock = 463;
            buttonTerm.Enabled = false;

            bool f = true;
            if (serialPort.IsOpen == true) serialPort.Close();
            serialOpen(comboBoxBaudWrite.SelectedItem.ToString());
            if (serialPort.IsOpen == true)
            {
                fWriting = true;
                try
                {
                    for (i = 0; i < 7; i++) serialPort.ReadByte();
                    f = true;
                }
                catch (TimeoutException ex)
                {
//                    MessageBox.Show(serialPort.BytesToRead.ToString());
//                    MessageBox.Show(ex.ToString());
                    MessageBox.Show("No response from PSoC1duino.\nPlease check connection.");
                    f = false;
                }
                if (f == true)
                {
                    serialPort.Write("OK!");
                    try
                    {
                        for (i = 0; i < 3; i++) serialPort.ReadByte();
                        f = true;
                    }
                    catch (TimeoutException exx)
                    {
                        MessageBox.Show(exx.ToString());
                        f = false;
                    }
                    if (f == true)
                    {
                        labelConnectStatus.Text = "Connected";
                        buttonOpenFile.Enabled = true;
                    }
                }
            }
            if (f == true)
            {
                Boolean fOpen = true;
                try
                {
                    sw = new StreamReader(fname);
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Failed to open HEX file."); ;
                    fOpen = false;
                }

                if (fOpen == true)
                {
                    sw.ReadLine();
                    progressBarWrite.Maximum = LastBlock;
                    labelConnectStatus.Text = "Writing";

                    for (b = 1; b <= LastBlock; b++)
                    {
                        progressBarWrite.Value = b;
                        ts = "S"; serialPort.Write(ts);
                        s = (sw.ReadLine()).ToLower(); // except terminal 0x0d+0x0a
                        // for HighSpeed (2012/12/19:akita)
                        // for (i = 1; i < 137; i++)
                        // {
                        //     serialPort.Write(s.Substring(i, 1));
                        // }
                        for (i = 1; i < 137; i += 2)
                        {
                            byte[] d = new byte[1];
                            d[0] = (byte)Convert.ToInt16(s.Substring(i, 2), 16);
                            serialPort.Write(d, 0, 1);
                        }
                        ts = "F"; serialPort.Write(ts);
                        try
                        {
                            serialPort.ReadByte();
                            // for HighSpeed (2012/12/19:akita)
                            // for (i = 1; i < 137; i++) serialPort.ReadByte();
                            // serialPort.ReadByte();
                            serialPort.ReadByte();
                        }
                        catch (TimeoutException exx)
                        {
                            //                    MessageBox.Show("Failed to download. Please re-program boot loader.");
                            MessageBox.Show("Failed to download. Please retry, or use MiniProg.\n\n" + exx.ToString());
                            b = LastBlock;
                        }
                    }
                    sw.Close();
                    serialOpen(comboBoxBaudTerm.SelectedItem.ToString());
                    serialPort.Close();
                    labelConnectStatus.Text = "Disconnected";
                    //            buttonTerm.Enabled = true;
                    fWriting = false;
                }
            }
        }

        // http://social.msdn.microsoft.com/Forums/ja-JP/csharpgeneralja/thread/e412787f-586b-40ea-aa60-e6857aa44d68        
        private void ReceiveByte(byte[] buf, int n)
        {
            char c;
            if (this.InvokeRequired == true)
            {
                this.Invoke(new Action<byte[], int>(this.ReceiveByte), buf, n);
                return;
            }
            if (textBoxTerm.Enabled == true)
            {
                for (int i = 0; i < n; i++)
                {
                    c = (char)buf[i];
                    if (fTermBinary == false) textBoxTermR.AppendText(c.ToString());
                    else
                    {
                        string s = String.Format("{0:X2}", buf[i]);
                        textBoxTermR.AppendText(s + " ");
                        nb++;
                        if (nb == 16)
                        {
                            nb = 0;
                            textBoxTermR.AppendText("\r\n");
                        }
                    }
                }
            }
        }

        private void serialPort_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            byte[] buf = new byte[4096];
            int n;
            n = serialPort.BytesToRead;
            if (fWriting == false)
            {
                serialPort.Read(buf, 0, n);
                this.ReceiveByte(buf, n);
                //            for (int i = 0; i < n; i++)
                //                this.ReceiveByte((Int32)buf[i]);
            }
        }
/*
        public delegate void myDelegate(int b);
        private void ReceiveByte(int b)
        {
            char c;
            if (textBoxTerm.Enabled == true)
            {
                c = (char)b;
                if (fTermBinary == false) textBoxTerm.AppendText(c.ToString());
                else
                {
                    string s = String.Format("{0:X2}", b);
                    textBoxTerm.AppendText(s + " ");
                }
            }
        }

        private void serialPort_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            byte[] buf = new byte[4096];
            int n;
            n = serialPort.BytesToRead;
            Console.WriteLine(n.ToString());
            myDelegate dlgByte = new myDelegate(ReceiveByte);
            serialPort.Read(buf, 0, n);
            for (int i = 0; i < n; i++)
                this.Invoke(dlgByte, (Int32)buf[i]);
        }
*/
        private void textBoxTerm_KeyUp(object sender, KeyEventArgs e)
        {
            serialPort.Write(textBoxTerm.Text.Substring(textBoxTerm.Text.Length - 1, 1));
        }

        private void comboBoxCOMport_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (serialPort.IsOpen == true) serialPort.Close();
            serialPort.PortName = comboBoxCOMport.Text.Substring(0, comboBoxCOMport.Text.IndexOf(" "));
//            serialOpen(comboBoxBaudTerm.SelectedItem.ToString());
            buttonTerm.Enabled = true;
            buttonWrite.Enabled = true;
        }

        private void checkBoxTermBinary_CheckedChanged(object sender, EventArgs e)
        {
            fTermBinary = checkBoxTermBinary.Checked;
            if (fTermBinary == true)
            {
                if (textBoxTerm.Enabled == true)
                {
                    buttonTermBinarySend.Enabled = true;
                    textBoxTermBinarySend.Enabled = true;
                    textBoxTermBinarySend.Focus();
                }
            }
            else
            {
                buttonTermBinarySend.Enabled = false;
                textBoxTermBinarySend.Enabled = false;
            }
        }

        private void buttonTermBinarySend_Click(object sender, EventArgs e)
        {
            string s;
            int i, n, d;
            s = textBoxTermBinarySend.Text;
            n = s.Length;
            if (n % 2 == 1) s += "0";
            for (i = 0; i < n; i += 2)
            {
                d = 0;
                try
                {
                    d = Convert.ToInt32(s.Substring(i, 2), 16);
                }
                catch (Exception ex)
                {
                    d = 0;
                }
                string ss = String.Format("{0:X2}", d);
                textBoxTerm.AppendText(ss+" ");
                byte[] buf = new byte[2];
                buf[0] = (byte)d;
                serialPort.Write(buf, 0, 1);
            }
            textBoxTermBinarySend.Text = "";
            textBoxTermBinarySend.Focus();
        }

        private void comboBoxBaudWrite_SelectedIndexChanged(object sender, EventArgs e)
        {
//            serialOpen(comboBoxBaudWrite.SelectedItem.ToString());
        }

        private void comboBoxBaudTerm_SelectedIndexChanged(object sender, EventArgs e)
        {
//            serialOpen(comboBoxBaudTerm.SelectedItem.ToString());
        }

        private void textBoxTermBinarySend_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Enter)
                buttonTermBinarySend_Click(sender, e);
        }

        private void buttonTermClear_Click(object sender, EventArgs e)
        {
            textBoxTerm.Clear();
            textBoxTermR.Clear();
        }

        private void Form1_Resize(object sender, EventArgs e)
        {
            ViewResize();
        }

        void ViewResize()
        {
            labelTermSend.Left = 10;
            labelTermReceive.Left = (this.Width - 30) / 2 + 20;
            buttonTermClear.Left = (this.Width - 30) / 2 + 20 - buttonTermClear.Size.Width;
            textBoxTerm.Left = 10;
            textBoxTerm.Height = this.Height - textBoxTerm.Location.Y - 30;
            textBoxTerm.Width = (this.Width - 30) / 2;
            textBoxTermR.Left = (this.Width - 30) / 2 + 20;
            textBoxTermR.Height = this.Height - textBoxTerm.Location.Y - 30;
            textBoxTermR.Width = (this.Width - 30) / 2;
        }
    }
}
